package hk.advanpro.android.sdk.device.ble;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;

import java.io.File;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.UUID;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.algorithm.belt.BLEBeltAlg;
import hk.advanpro.android.sdk.algorithm.belt.BLEBeltBreathAnalyzer;
import hk.advanpro.android.sdk.algorithm.belt.BLEBeltSleepAnalyzer;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.callback.DeviceUpgradeCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.device.ota.advanpro.FirmwareUpgrade;
import hk.advanpro.android.sdk.device.params.DeviceCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltNotifyBreathCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltNotifySleepCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltNotifyBreathCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltNotifySleepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltRealSensorEventResult;

import static hk.advanpro.android.sdk.device.protocol.BLEBeltProtocol.*;

/**
 * Created by yejianfei on 2017/6/15.
 */

public class BLEBeltDevice extends DefaultBLEDevice {

    /**
     * 监听实时拉力,三轴数据
     */
    public static final String EVENT_BELT_REAL_SENSOR = "EVENT_BELT_REAL_SENSOR";

    /**
     * 监听实时心率
     */
    public static final String EVENT_BELT_HEART_RATE = "EVENT_BELT_HEART_RATE";

    /**
     * 监听步数
     */
    public static final String EVENT_BELT_NOTIFY_STEP = "EVENT_BELT_NOTIFY_STEP";

    /**
     * 呼吸分析事件
     */
    public static final String EVENT_BELT_NOTIFY_BREATH = "EVENT_BELT_NOTIFY_BREATH";

    /**
     * 睡眠分析事件
     */
    public static final String EVENT_BELT_NOTIFY_SLEEP = "EVENT_BELT_NOTIFY_SLEEP";

    /**
     * 获取某天步数命令
     */
    public static final String CMD_BELT_STEP = "CMD_BELT_STEP";

    /**
     * 获取某天分段步数命令
     */
    public static final String CMD_BELT_SECTION_STEP = "CMD_BELT_SECTION_STEP";

    /**
     * 开启实时步数命令
     */
    public static final String CMD_BELT_NOTIFY_STEP = "CMD_BELT_NOTIFY_STEP";

    /**
     * 开始数据存储命令
     */
    public static final String CMD_BELT_STORAGE_START = "CMD_BELT_STORAGE_START";

    /**
     * 关闭数据存储命令
     */
    public static final String CMD_BELT_STORAGE_STOP = "CMD_BELT_STORAGE_STOP";

    /**
     * 查询数据存储状态命令
     */
    public static final String CMD_BELT_STORAGE_QUERY = "CMD_BELT_STORAGE_QUERY";

    /**
     * 清除数据存储命令
     */
    public static final String CMD_BELT_STORAGE_CLEAR = "CMD_BELT_STORAGE_CLEAR";

    /**
     * 开启或关闭呼吸分析命令
     */
    public static final String CMD_BELT_NOTIFY_BREATH = "CMD_BELT_NOTIFY_BREATH";

    /**
     * 开启或关闭睡眠分析命令
     */
    public static final String CMD_BELT_NOTIFY_SLEEP = "CMD_BELT_NOTIFY_SLEEP";

    private static final UUID PRIMARY_SERVICE_UUID = UUID.fromString("00008001-0000-1000-8000-00805F9B34FB");

    private static final UUID RESISTER_CHARACTERISTIC_CHARACTERISTIC = UUID.fromString("00008002-0000-1000-8000-00805F9B34FB");

    private static final UUID ACC_X_CHARACTERISTIC_CHARACTERISTIC = UUID.fromString("00008003-0000-1000-8000-00805F9B34FB");

    private static final UUID ACC_Y_CHARACTERISTIC_CHARACTERISTIC = UUID.fromString("00008004-0000-1000-8000-00805F9B34FB");

    private static final UUID ACC_Z_CHARACTERISTIC_CHARACTERISTIC = UUID.fromString("00008005-0000-1000-8000-00805F9B34FB");

    private static final UUID RANGE_CHARACTERISTIC_CHARACTERISTIC = UUID.fromString("00008006-0000-1000-8000-00805F9B34FB");

    private static final UUID HEAR_RATE_SERVICE_UUID = UUID.fromString("0000180D-0000-1000-8000-00805F9B34FB");

    private static final UUID HEAR_RATE_CHARACTERISTIC = UUID.fromString("00002A37-0000-1000-8000-00805F9B34FB");

    private static final UUID BELT_STEP_CHARACTERISTIC = UUID.fromString("0000808F-0000-1000-8000-00805F9B34FB");

    private static final String TAG = BLEBeltDevice.class.getSimpleName();

    private ByteBuffer buffer = ByteBuffer.allocate(21);

    private ByteBuffer sections = ByteBuffer.allocate(20);

    /**
     * 加速度量程数据,此数据在设备运行过程中不会更改
     */
    private int accRange = 0;

    /**
     * 柔性传电阻量程数据,此数据在设备运行过程中不会更改
     */
    private int range = 0;

    /**
     * 柔性传感器数据 20bytes
     */
    private byte[] resister = new byte[20];

    /**
     * X轴加速度信息 20bytes
     */
    private byte[] accX = new byte[20];

    /**
     * Y轴加速度信息 20bytes
     */
    private byte[] accY = new byte[20];

    /**
     * Z轴加速度信息 20bytes
     */
    private byte[] accZ = new byte[20];

    private BLEBeltBreathAnalyzer breathAnalyzer = new BLEBeltBreathAnalyzer();
    private BLEBeltSleepAnalyzer sleepAnalyzer = new BLEBeltSleepAnalyzer();

    private int sleepAnalyseResultInterval = 0;//睡眠分析时间间隔（秒），为0时表示中途不分析，默认为0
    private long lastSleepAnalyzerResultTime = 0;//上一次分析睡眠时间

    private final DefaultBluetoothGattCallback callback = new DefaultBluetoothGattCallback() {

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            super.onServicesDiscovered(gatt, status);
            if (DeviceMode.Application.name().equals(getInfo(BLE_DEVICE_INFO_MODE))) {
                // 读取心率、电阻值
                read(PRIMARY_SERVICE_UUID, RANGE_CHARACTERISTIC_CHARACTERISTIC);
                BLEBeltDevice.this.notify(PRIMARY_SERVICE_UUID, RESISTER_CHARACTERISTIC_CHARACTERISTIC, true);

                //监听三轴通道
                if (getBluetoothGattCharacteristic(PRIMARY_SERVICE_UUID, ACC_X_CHARACTERISTIC_CHARACTERISTIC) != null)
                    BLEBeltDevice.this.notify(PRIMARY_SERVICE_UUID, ACC_X_CHARACTERISTIC_CHARACTERISTIC, true);
                if (getBluetoothGattCharacteristic(PRIMARY_SERVICE_UUID, ACC_Y_CHARACTERISTIC_CHARACTERISTIC) != null)
                    BLEBeltDevice.this.notify(PRIMARY_SERVICE_UUID, ACC_Y_CHARACTERISTIC_CHARACTERISTIC, true);
                if (getBluetoothGattCharacteristic(PRIMARY_SERVICE_UUID, ACC_Z_CHARACTERISTIC_CHARACTERISTIC) != null)
                    BLEBeltDevice.this.notify(PRIMARY_SERVICE_UUID, ACC_Z_CHARACTERISTIC_CHARACTERISTIC, true);
                //监听心率通道
                if (getBluetoothGattCharacteristic(HEAR_RATE_SERVICE_UUID, HEAR_RATE_CHARACTERISTIC) != null)
                    BLEBeltDevice.this.notify(HEAR_RATE_SERVICE_UUID, HEAR_RATE_CHARACTERISTIC, true);
            }

        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);

            if (RESISTER_CHARACTERISTIC_CHARACTERISTIC.equals(characteristic.getUuid())) {
                resister = characteristic.getValue();
                BLEBeltRealSensorEventResult sensorResult = buildRealSensorResult(resister, accX, accY, accZ, range, accRange);
                fireEvent(EVENT_BELT_REAL_SENSOR, sensorResult);
                if (breathAnalyzer.isEnable() && breathAnalyzer.push(sensorResult)) {
                    BLEBeltAlg.SmartBelt_TSBreathStats breathStats = breathAnalyzer.poll();
                    if (breathStats != null)
                        fireEvent(EVENT_BELT_NOTIFY_BREATH, buildBreathEventResult(breathStats));
                }
                if (sleepAnalyzer.isEnable()) {
                    for (int i = 0; i < sensorResult.getAccXData().length; i++) {
                        if (!sleepAnalyzer.push(sensorResult.getAccXData()[i], sensorResult.getAccYData()[i], sensorResult.getAccZData()[i])) {
                            continue;
                        }
                    }
                    if (sleepAnalyseResultInterval > 0 && System.currentTimeMillis() - lastSleepAnalyzerResultTime >= sleepAnalyseResultInterval * 1000) {
                        lastSleepAnalyzerResultTime = System.currentTimeMillis();
                        BLEBeltAlg.SleepAnalyzerOut breathStats = sleepAnalyzer.poll();
                        if (breathStats != null)
                            fireEvent(EVENT_BELT_NOTIFY_SLEEP, buildSleepEventResult(breathStats));
                    }

                }
            } else if (ACC_X_CHARACTERISTIC_CHARACTERISTIC.equals(characteristic.getUuid())) {
                accX = characteristic.getValue();
            } else if (ACC_Y_CHARACTERISTIC_CHARACTERISTIC.equals(characteristic.getUuid())) {
                accY = characteristic.getValue();
            } else if (ACC_Z_CHARACTERISTIC_CHARACTERISTIC.equals(characteristic.getUuid())) {
                accZ = characteristic.getValue();
            } else if (HEAR_RATE_CHARACTERISTIC.equals(characteristic.getUuid())) {// 心率数据
                fireEvent(EVENT_BELT_HEART_RATE, buildHeartRateResult(characteristic.getValue()));
            } else if (BELT_STEP_CHARACTERISTIC.equals(characteristic.getUuid())) {
                byte[] data = characteristic.getValue();

                byte head = data[0];
                byte flag = data[1];

                short key = (short) (((head << 8) & 0xffff) | (flag & 0xff));

                switch (key) {
                    case 0x0201:
                    case 0x0202:
                        // 统一计步数据新就版本格式差异的问题。
                        ByteBuffer temp = ByteBuffer.allocate(21);
                        if (data.length == 13) {
                            temp.put(Arrays.copyOfRange(data, 0, 8));
                            temp.put(new byte[]{0x00, 0x00, 0x00, 0x00});
                            temp.put(Arrays.copyOfRange(data, 8, 12));
                            temp.put(new byte[]{0x00, 0x00, 0x00, 0x00});
                            temp.put(Arrays.copyOfRange(data, 12, 13));

                            data = temp.array();
                        }

                        if (data.length == 17) {
                            temp.put(Arrays.copyOfRange(data, 0, 7));
                            temp.put((byte) 0x00);
                            temp.put(Arrays.copyOfRange(data, 7, 10));
                            temp.put((byte) 0x00);
                            temp.put(Arrays.copyOfRange(data, 10, 13));
                            temp.put((byte) 0x00);
                            temp.put(Arrays.copyOfRange(data, 13, 16));
                            temp.put((byte) 0x00);
                            temp.put(Arrays.copyOfRange(data, 16, 17));

                            data = temp.array();
                        }

                        if (buffer.remaining() < 21) {
                            byte[] bb = buffer.array();
                            buffer = ByteBuffer.allocate(bb.length + 21);
                            buffer.put(bb);
                        }

                        buffer.put(data);

                        if (data[data.length - 1] == 0x00) {
                            byte[] bb = buffer.array();
                            buffer = ByteBuffer.allocate(21);
                            fireCommand(CMD_BELT_STEP, buildStepResult(bb));
                        }
                        break;
                    case 0x0203:
                        if (data.length == 4) {
                            fireCommand(CMD_BELT_NOTIFY_STEP, buildNotifyStepResult(data));

                        } else {
                            fireEvent(EVENT_BELT_NOTIFY_STEP, buildNotifyStepEventResult(data));
                        }
                        break;
                    case 0x0242:
                        if (sections.remaining() < 20) {
                            byte[] bb = sections.array();
                            sections = ByteBuffer.allocate(bb.length + 20);
                            sections.put(bb);
                        }

                        sections.put(data);
                        if (data[data.length - 1] >> 8 == 0x00) {
                            byte[] bb = sections.array();
                            sections = ByteBuffer.allocate(20);
                            fireCommand(CMD_BELT_SECTION_STEP, buildSectionStepResult(bb));
                        }
                        break;
                    case 0x0500:
                        fireCommand(CMD_BELT_STORAGE_QUERY, buildStorageQueryResult(data));
                        break;
                    case 0x0501:
                        fireCommand(CMD_BELT_STORAGE_START, buildStorageStartResult(data));
                        break;
                    case 0x0546:
                        fireCommand(CMD_BELT_STORAGE_CLEAR, buildStorageClearResult(data));
                        break;
                    case 0x05ff:
                        fireCommand(CMD_BELT_STORAGE_STOP, buildStorageStopResult(data));
                        break;
                    default:
                }
            }
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            super.onCharacteristicRead(gatt, characteristic, status);

            if (characteristic.getUuid().equals(RANGE_CHARACTERISTIC_CHARACTERISTIC)) {
                byte[] data = characteristic.getValue();//5bytes(1byte ACC_RANGE + 4bytes R0
                if (data != null && data.length >= 5) {
                    accRange = data[0];
                    int rangeVal = (data[1] & 0xff) + (data[2] << 8) + (data[3] << 16) + (data[4] << 24);
                    if (rangeVal > 0 && rangeVal < 9000000) range = rangeVal;
                }
            }
        }
    };

    @Override
    protected CommandChannelInfo getCommandChannelInfo(String key) {
        CommandChannelInfo info = super.getCommandChannelInfo(key);
        if (info == null) {
            UUID uuid;

            switch (key) {
                case CMD_BELT_STEP:
                case CMD_BELT_NOTIFY_STEP:
                case CMD_BELT_SECTION_STEP:
                    uuid = BELT_STEP_CHARACTERISTIC;
                    break;
                default:
                    uuid = BELT_STEP_CHARACTERISTIC;
            }

            info = new CommandChannelInfo(key, PRIMARY_SERVICE_UUID, uuid);
        }

        return info;
    }

    @Override
    public synchronized void command(String key, DeviceCommandParams params, DeviceCommandCallback cb) {
        this.command(key, params, 30, cb);
    }

    @Override
    public void command(String key, DeviceCommandParams params, int timeOut, DeviceCommandCallback cb) {
        if (("C2").equalsIgnoreCase(getInfo(BLE_DEVICE_INFO_COMPANY))
                && (CMD_BELT_STEP.equals(key) || CMD_BELT_SECTION_STEP.equals(key) || CMD_BELT_STORAGE_START.equals(key) ||
                CMD_BELT_STORAGE_STOP.equals(key) || CMD_BELT_STORAGE_QUERY.equals(key) || CMD_BELT_STORAGE_CLEAR.equals(key))) {
            cb.onError(BLEBeltDevice.this, new DeviceCallbackException(ERROR_FUNCTION_NOT_SUPPORT, "key=" + key));
            return;
        }

        if (CMD_BELT_NOTIFY_BREATH.equals(key)) {
            try {
                BLEBeltNotifyBreathCommandParams params1 = (BLEBeltNotifyBreathCommandParams) params;
                if (params1.isEnable()) {
                    breathAnalyzer.init();
                } else {
                    breathAnalyzer.exit();
                }
                cb.onSuccess(BLEBeltDevice.this, new BLEBeltNotifyBreathCommandResult(breathAnalyzer.isEnable() == params1.isEnable()));
            } catch (Exception e) {
                cb.onError(BLEBeltDevice.this, new DeviceCallbackException(e));
            }
            return;
        }
        if (CMD_BELT_NOTIFY_SLEEP.equals(key)) {
            try {
                BLEBeltNotifySleepCommandParams params1 = (BLEBeltNotifySleepCommandParams) params;
                if (params1.isEnable()) {
                    sleepAnalyzer.init();
                    sleepAnalyseResultInterval = params1.getInterval();
                    cb.onSuccess(BLEBeltDevice.this, new BLEBeltNotifySleepCommandResult(sleepAnalyzer.isEnable() == params1.isEnable()));
                } else {
                    BLEBeltAlg.SleepAnalyzerOut breathStats = sleepAnalyzer.poll();
                    if (breathStats != null)
                        fireEvent(EVENT_BELT_NOTIFY_SLEEP, buildSleepEventResult(breathStats)); //返回分析结果
                    sleepAnalyzer.exit();//退出睡眠分析
                    cb.onSuccess(BLEBeltDevice.this, new BLEBeltNotifySleepCommandResult(sleepAnalyzer.isEnable() == params1.isEnable()));
                }
            } catch (Exception e) {
                cb.onError(BLEBeltDevice.this, new DeviceCallbackException(e));
            }
            return;
        }
        super.command(key, params, timeOut, cb);
    }

    @Override
    protected void afterRegisterCommand(String key, DeviceCommandCallback cb) {

    }

    @Override
    protected void onCommandTimeOut(String key) {

    }

    @Override
    public void start(DeviceCommandParams params, DeviceCommandCallback cb) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void stop(DeviceCommandParams params, DeviceCommandCallback cb) {
        throw new UnsupportedOperationException();
    }

    private boolean isUpgrading = false;

    @Override
    public boolean upgrade(final File file, final DeviceUpgradeCallback cb) {
        if (isUpgrading()) {
            cb.onError(BLEBeltDevice.this, new DeviceCallbackException(Device.ERROR_ALREADY_UPGRADE));
            return false;
        }
        if (!file.exists()) {
            cb.onError(BLEBeltDevice.this, new DeviceCallbackException(Device.ERROR_UPGRADE_FILE_NOT_EXISTS));
            return false;
        }
        if (!isConnected()) {
            //升级之前先连接设备
            cb.onError(BLEBeltDevice.this, new DeviceCallbackException(Device.ERROR_IS_DISCONNECTED));
            return false;
        }
        isUpgrading = true;
        //先断开连接
        this.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                FirmwareUpgrade.doUpgrade(BLEBeltDevice.this.result.getContext(), getInfo(BLE_DEVICE_INFO_ADDRESS), file, new FirmwareUpgrade.Callback() {
                    @Override
                    public void onError(DeviceCallbackException err) {
                        isUpgrading = false;
                        cb.onError(BLEBeltDevice.this, err);
                    }

                    @Override
                    public void onSuccess() {
                        isUpgrading = false;
                        //升级完成
                        //重新扫描设备
                        BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
                        manager.instance(new DeviceManager.DeviceConnection(result.getAddress()), 30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {
                            @Override
                            public void onScanning(BLEDeviceScanResult result) {
                                //打开设备
                                open(result);
                                //连接设备
                                connect();
                            }
                            void connect() {
                                BLEBeltDevice.this.connect(new DeviceConnectCallback() {
                                    @Override
                                    public void onSucceed(Device device, DeviceConnectResult result2) {
                                        //设备连接成功
                                        cb.onSuccess(BLEBeltDevice.this, null);
                                    }

                                    @Override
                                    public void onError(Device device, DeviceCallbackException error) {
                                        //设备连接失败，升级失败
                                        cb.onError(device, error);
                                    }
                                });
                            }

                            @Override
                            public void onStop() {

                            }

                            @Override
                            public void onError(DeviceCallbackException error) {
                                cb.onError(BLEBeltDevice.this, error);
                            }
                        });
                    }

                    @Override
                    public void onProgress(long total, long progress) {
                        cb.onProgress(BLEBeltDevice.this, total, progress);
                    }
                });
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                isUpgrading = false;
                cb.onError(device, error);
            }
        });
        return true;
    }

    @Override
    public boolean isUpgrading() {
        return isUpgrading;
    }

    @Override
    public void settings(DeviceCommandParams params, DeviceCommandCallback cb) {
        throw new UnsupportedOperationException();
    }

    @Override
    protected BluetoothGattCallback getBluetoothGattCallback() {
        return callback;
    }

}
