package hk.advanpro.android.sdk.device.ble;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.enumeration.ConnectType;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.utils.CommonUtils;

/**
 * 低耗功蓝牙设备管理类
 * Created by yejianfei on 2017/6/7.
 */
@SuppressWarnings("deprecation")
public class BLEDeviceManager implements DeviceManager<BLEDeviceScanResult> {

    private static final String TAG = BLEDeviceManager.class.getSimpleName();

    private Context context;

    private BluetoothManager bluetoothManager;

    private BluetoothAdapter bluetoothAdapter;

    private Set<BLEDeviceScanResult.ScanRecord> records = new LinkedHashSet<>();

    private DeviceManagerScanCallback<BLEDeviceScanResult> scanCallback;
    private DeviceType deviceType;

    private boolean scanning = false;
    private Timer timer;


    private BluetoothAdapter.LeScanCallback callback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public void onLeScan(BluetoothDevice device, int rssi, byte[] scanRecord) {
            ALog.d(TAG, String.format("ls scan device name %s [%s] and rssi %d scanRecord %s", device.getName(), device.getAddress(), rssi, CommonUtils.bytesToHexString(scanRecord)));
            BLEDeviceScanResult.ScanRecord record = createScanRecord(device, rssi, scanRecord);
            if (scanning && record != null && !records.contains(record) && (deviceType == null || deviceType.equals(record.getManufacturer().getType()))) {
                records.add(record);
                BLEDeviceScanResult result = new BLEDeviceScanResult(context, record, device);
                scanCallback.onScanning(result);

            }

        }
    };

    public Context getContext() {
        return context;
    }

    public BluetoothManager getBluetoothManager() {
        return bluetoothManager;
    }

    public BluetoothAdapter getBluetoothAdapter() {
        return bluetoothAdapter;
    }

    public synchronized void init(Context context) {
        this.context = context;
        bluetoothManager = (BluetoothManager) context.getSystemService(Context.BLUETOOTH_SERVICE);
        bluetoothAdapter = bluetoothManager.getAdapter();
    }

    /**
     * 进行设备扫描，蓝牙使用系统扫描机制，串口获取系统串口列表。
     *
     * @param timeout 超时时间 单位秒
     * @param cb      回调接口
     */
    @Override
    public synchronized void scan(final int timeout, final DeviceManagerScanCallback<BLEDeviceScanResult> cb) {
        this.scan(timeout, null, cb);
    }

    @Override
    public void scan(int timeout, DeviceType filterType, DeviceManagerScanCallback<BLEDeviceScanResult> cb) {
        this.scanCallback = cb;
        this.deviceType = filterType;

        if (timeout > 0) {
            timer = new Timer();
            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    if (scanning) {
                        stop();
                    }

                }
            }, timeout * 1000);
        }
        scanning = true;
        bluetoothAdapter.startLeScan(callback);
    }

    @Override
    public synchronized void stop() {
        scanning = false;
        bluetoothAdapter.stopLeScan(callback);
        if (timer != null) {
            timer.cancel();
            timer = null;
        }
        if (scanCallback != null) {
            scanCallback.onStop();
            scanCallback = null;
        }
        records.clear();
    }

    @Override
    public void instance(final DeviceConnection connection, final DeviceManagerScanCallback<BLEDeviceScanResult> cb) {
        instance(connection, 30, cb);
    }

    @Override
    public void instance(final DeviceConnection connection, final int timeout, final DeviceManagerScanCallback<BLEDeviceScanResult> cb) {
        final Timer timer = new Timer();
        final BluetoothAdapter.LeScanCallback callback = new BluetoothAdapter.LeScanCallback() {
            boolean isStop = false;

            @Override
            public void onLeScan(BluetoothDevice device, int rssi, byte[] scanRecord) {
                ALog.d(TAG, String.format("instance ls scan device name %s [%s] and rssi %d scanRecord %s", device.getName(), device.getAddress(), rssi, CommonUtils.bytesToHexString(scanRecord)));
                if (!isStop && device.getAddress().equals(connection.getAddress())) {
                    isStop = true;
                    timer.cancel();
                    bluetoothAdapter.stopLeScan(this);

                    BLEDeviceScanResult.ScanRecord record = createScanRecord(device, rssi, scanRecord);
                    BLEDeviceScanResult result = new BLEDeviceScanResult(context, record, device);
                    cb.onScanning(result);
                    cb.onStop();
                }
            }
        };
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                cb.onError(new DeviceCallbackException(Device.ERROR_IS_TIME_OUT, "timeout=" + timeout + "s"));
                bluetoothAdapter.stopLeScan(callback);
            }
        }, timeout * 1000);
        bluetoothAdapter.startLeScan(callback);
    }

    protected BLEDeviceScanResult.ScanRecord createScanRecord(BluetoothDevice device, int rssi, byte[] record) {
        short flag = 0;
        short uuid = 0;
        String localName = null;
        byte[] manufacturer = null;

        ByteBuffer buffer = ByteBuffer.wrap(record).order(ByteOrder.LITTLE_ENDIAN);
        while (buffer.remaining() > 2) {
            byte len = buffer.get();

            if (len <= 0 || len > buffer.remaining()) {
                break;
            }

            byte type = buffer.get();
            byte[] val = new byte[len - 1];

            switch (type) {
                case 0x01:
                    flag = buffer.get();
                    break;
                case 0x02:
                    uuid = buffer.getShort();
                    break;
                case 0x03:
                    uuid = buffer.getShort();
                    break;
                case 0x09:
                    buffer.get(val);
                    localName = new String(val);
                    break;
                case (byte) 0xff:
                    buffer.get(val);
                    manufacturer = val;
                    break;
                default:
                    buffer.get(val);
                    break;

            }
        }

        if ((manufacturer == null || (manufacturer[0] != (byte) 0xc1 && manufacturer[0] != (byte) 0xc2))) {
            return null;
        } else {
            BLEDeviceScanResult.ScanRecord info = new BLEDeviceScanResult.ScanRecord();
            info.setUuid(uuid);
            info.setLocalName(localName);
            info.setRssi(rssi);
            info.setAddress(device.getAddress());

            if (manufacturer != null) {
                BLEDeviceScanResult.ScanRecordManufacturer data = new BLEDeviceScanResult.ScanRecordManufacturer();
                data.setCompany(String.format("%02x", (short) (manufacturer[0] & 0xff)).toUpperCase());
                int type = (manufacturer[1] & 0xff);
                DeviceType deviceType = DeviceType.Unknown;
                switch (type) {
                    case 1:
                        deviceType = DeviceType.Belt;// 呼吸带
                        break;
                    case 2:
                        deviceType = DeviceType.Band;// 腰带
                        break;
                    case 3:
                        deviceType = DeviceType.SMan;// 内裤
                        break;
                    case 4:
                        deviceType = DeviceType.Insole; // 鞋垫
                        break;
                    case 7:
                        deviceType = DeviceType.Futon;// 跪垫
                        break;
                    case 8:
                        deviceType = DeviceType.Kedian; // 磕垫
                        break;
                    case 10:
                        deviceType = DeviceType.SDP; // 开发版
                        break;
                    case 11:
                        deviceType = DeviceType.SDP_Inaole; // 开发版鞋垫
                        break;
                }
                data.setType(deviceType);
                String address = String.format("%02x", manufacturer[3]) + ":" +
                        String.format("%02x", manufacturer[4]) + ":" +
                        String.format("%02x", manufacturer[5]) + ":" +
                        String.format("%02x", manufacturer[6]) + ":" +
                        String.format("%02x", manufacturer[7]) + ":" +
                        String.format("%02x", manufacturer[8]);
                data.setAddress(address.toUpperCase());

                byte product = (byte) (manufacturer[2] & 0xff);
                data.setMode((product >> 7) == 0 ? DeviceMode.Application : DeviceMode.BootLoader);
                data.setVersion((short) ((product >> 3) & 0x7));
                data.setMajor((short) (product & 0x7));

                HashMap<String, String> map = new HashMap<>();
                switch (data.getType()) {
                    case Insole:
                    case SDP_Inaole:
                        map.put("foot", String.valueOf((product >> 6)));//0左脚 1右脚
                        break;
                }

                data.setSerial(String.format("%02x%02x%02x", manufacturer[0], manufacturer[1], manufacturer[2]).toUpperCase());
                data.setDescription(map);
                info.setManufacturer(data);
            }

            return info;
        }
    }

    @Override
    public boolean isEnable(ConnectType type) {
        switch (type) {
            case BLE:
                return bluetoothManager != null && bluetoothAdapter != null && bluetoothAdapter.enable();
        }
        return false;
    }

    @Override
    public boolean isScanning() {
        return scanning;
    }
}
