package hk.advanpro.android.sdk.device.ble;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.utils.CommonUtils;

import static hk.advanpro.android.sdk.device.protocol.BLEProtocol.buildEnableWritePermissioneResult;
import static hk.advanpro.android.sdk.device.protocol.BLEProtocol.buildSendPermissionResult;
import static hk.advanpro.android.sdk.device.protocol.BLEProtocol.buildWriteDeviceCodeResult;
import static hk.advanpro.android.sdk.device.protocol.BLEProtocol.buildWriteDeviceIdResult;

/**
 * 低耗功蓝牙设备基类
 * Created by yejianfei on 2017/6/7.
 */
@SuppressWarnings("unchecked")
abstract public class DefaultBLEDevice extends BLEDevice {

    public static final String EVENT_BATTERY_CHANGE = "EVENT_BATTERY_CHANGE";

    public static final String CMD_ENABLE_WRITE_PERMISSION = "CMD_ENABLE_WRITE_PERMISSION";

    public static final String CMD_SEND_PERMISSION_CODE = "CMD_SEND_PERMISSION_CODE";

    public static final String CMD_WRITE_DEVICE_CODE = "CMD_WRITE_DEVICE_CODE";

    public static final String CMD_WRITE_DEVICE_ID = "CMD_WRITE_DEVICE_ID";


    protected static final UUID INSOLE_BATTERY_SERVICE_UUID = UUID.fromString("0000180F-0000-1000-8000-00805F9B34FB");

    protected static final UUID BELT_BATTERY_SERVICE_UUID = UUID.fromString("00008001-0000-1000-8000-00805F9B34FB");

    protected static final UUID PIPE_SERVICE_UUID = UUID.fromString("00008001-0000-1000-8000-00805F9B34FB");

    protected static final UUID BATTERY_CHARACTERISTIC_UUID = UUID.fromString("00002A19-0000-1000-8000-00805F9B34FB");

    protected static final UUID INFO_SERVICE_UUID = UUID.fromString("0000180A-0000-1000-8000-00805F9B34FB");

    protected static final UUID INFO_CHARACTERISTIC_UUID = UUID.fromString("00002A24-0000-1000-8000-00805F9B34FB");

    protected static final UUID CHECKCODE_CHARACTERISTIC_UUID = UUID.fromString("00002A25-0000-1000-8000-00805F9B34FB");

    protected static final UUID FIRMWARE_CHARACTERISTIC_UUID = UUID.fromString("00002A26-0000-1000-8000-00805F9B34FB");

    protected static final UUID HARDWARE_CHARACTERISTIC_UUID = UUID.fromString("00002A27-0000-1000-8000-00805F9B34FB");

    protected static final UUID PIPE_CHARACTERISTIC_CHARACTERISTIC = UUID.fromString("0000808F-0000-1000-8000-00805F9B34FB");

    private static final String TAG = BLEDevice.class.getSimpleName();
    private static final Map<Short, String> COMMAND_KEYS = new HashMap<Short, String>() {{
        put((short) 0xffa1, CMD_ENABLE_WRITE_PERMISSION);
        put((short) 0xffb1, CMD_SEND_PERMISSION_CODE);
        put((short) 0x0311, CMD_WRITE_DEVICE_ID);
        put((short) 0x0312, CMD_WRITE_DEVICE_CODE);
    }};
    private BluetoothGattCallback callback = new DefaultBluetoothGattCallback();
    private boolean isInfoRead = false;

    @Override
    public boolean isInfoRead() {
        return isInfoRead;
    }

    @Override
    public synchronized void disconnect(DeviceDisconnectCallback cb) {
        super.disconnect(cb);
    }

    @Override
    public void connect(int timeOut, DeviceConnectCallback cb) {
        //修改设备信息未读取
        isInfoRead = false;
        super.connect(timeOut, cb);
    }

    @Override
    protected CommandChannelInfo getCommandChannelInfo(String key) {
        CommandChannelInfo info = null;

        switch (key) {
            case CMD_ENABLE_WRITE_PERMISSION:
                // 强制使用head + flag模式，原有协议为0xff0101
                info = new CommandChannelInfo(CMD_ENABLE_WRITE_PERMISSION, PIPE_SERVICE_UUID, PIPE_CHARACTERISTIC_CHARACTERISTIC);
                break;
            case CMD_SEND_PERMISSION_CODE:
                // 强制使用head + flag模式，原有协议为0xff0101
                info = new CommandChannelInfo(CMD_SEND_PERMISSION_CODE, PIPE_SERVICE_UUID, PIPE_CHARACTERISTIC_CHARACTERISTIC);
                break;
            case CMD_WRITE_DEVICE_CODE:
                info = new CommandChannelInfo(CMD_WRITE_DEVICE_CODE, PIPE_SERVICE_UUID, PIPE_CHARACTERISTIC_CHARACTERISTIC);
                break;
            case CMD_WRITE_DEVICE_ID:
                info = new CommandChannelInfo(CMD_WRITE_DEVICE_ID, PIPE_SERVICE_UUID, PIPE_CHARACTERISTIC_CHARACTERISTIC);
                break;
            default:
        }
        return info;
    }

    @Override
    protected BluetoothGattCallback getBluetoothGattCallback() {
        return callback;
    }

    public class DefaultBluetoothGattCallback extends BLEDeviceBluetoothGattCallback {

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            super.onServicesDiscovered(gatt, status);

            if (status == BluetoothGatt.GATT_SUCCESS) {
                if (DeviceMode.BootLoader.name().equals(getInfo(BLE_DEVICE_INFO_MODE))) {
                    //ota模式自动连接成功
                    fireConnected(null);
                } else if (!isFindService(PIPE_SERVICE_UUID) ||
                        !isFindService(INFO_SERVICE_UUID) ||
                        !isFindService(BELT_BATTERY_SERVICE_UUID)) {
                    //服务不存在
                    fireConnected(new DeviceCallbackException(ERROR_SERVICE_NOT_EXISTS));
                } else {
                    DefaultBLEDevice.this.notify(PIPE_SERVICE_UUID, PIPE_CHARACTERISTIC_CHARACTERISTIC, true);

                    BluetoothGattCharacteristic bluetoothGattCharacteristic
                            = getBluetoothGattCharacteristic(BELT_BATTERY_SERVICE_UUID, BATTERY_CHARACTERISTIC_UUID);
                    // 开启电量改变通道
                    if (bluetoothGattCharacteristic == null) {
                        DefaultBLEDevice.this.notify(INSOLE_BATTERY_SERVICE_UUID, BATTERY_CHARACTERISTIC_UUID, true);
                    } else {
                        DefaultBLEDevice.this.notify(BELT_BATTERY_SERVICE_UUID, BATTERY_CHARACTERISTIC_UUID, true);
                    }
                    //连接成功
                    fireConnected(null);
                    //读取设备信息
                    readDeviceInfo();
                }
            }

        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);

            ALog.d(TAG, String.format("[%s] notify %s & %s [%s]", bluetoothDevice.getAddress(), characteristic.getService().getUuid().toString(), characteristic.getUuid().toString(), CommonUtils.bytesToHexString(characteristic.getValue())));
            if (PIPE_CHARACTERISTIC_CHARACTERISTIC.equals(characteristic.getUuid())) {

                byte[] data = characteristic.getValue();
                byte head = data[0];
                byte flag = data[1];
                byte cmd = data[2];

                short commandKey = (short) (((head << 8) & 0xffff) | (flag & 0xff));

                if (head == (byte) 0xff && flag == (byte) 0x01 && cmd == (byte) 0x01) {
                    commandKey = (short) 0xffa1;
                } else if (head == (byte) 0xff && flag == (byte) 0x01 && cmd == (byte) 0x02) {
                    commandKey = (short) 0xffb1;
                }

                switch (commandKey) {
                    case (short) 0xffa1:
                        //提请开启权限命令结果
                        fireCommand(COMMAND_KEYS.get(commandKey), buildEnableWritePermissioneResult(data));
                        break;
                    case (short) 0xffb1:
                        //发送验证数据结果
                        fireCommand(COMMAND_KEYS.get(commandKey), buildSendPermissionResult(data));
                        break;
                    case (short) 0x0311:
                        //写入设备ID结果
                        fireCommand(COMMAND_KEYS.get(commandKey), buildWriteDeviceIdResult(data));
                        break;
                    case (short) 0x0312:
                        //写入用户ID结果
                        fireCommand(COMMAND_KEYS.get(commandKey), buildWriteDeviceCodeResult(data));
                        break;
                    default:
                        break;
                }
            }
            if (BATTERY_CHARACTERISTIC_UUID.equals(characteristic.getUuid())) {
                String oldString = info.get(BLE_DEVICE_INFO_BATTERY);
                String newString = String.valueOf(characteristic.getIntValue(BluetoothGattCharacteristic.FORMAT_SINT8, 0));
                if (!newString.equals(oldString)) {
                    DefaultBLEDevice.this.info.put(BLE_DEVICE_INFO_BATTERY, newString);
                    fireEvent(EVENT_BATTERY_CHANGE, null);
                }
            }
        }


        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            super.onCharacteristicRead(gatt, characteristic, status);

            if (characteristic.getUuid().equals(INFO_CHARACTERISTIC_UUID)) {
                DefaultBLEDevice.this.info.put(BLE_DEVICE_INFO_ID, new String(characteristic.getValue()));
            }

            if (characteristic.getUuid().equals(CHECKCODE_CHARACTERISTIC_UUID)) {
                DefaultBLEDevice.this.info.put(BLE_DEVICE_INFO_CODE, new String(characteristic.getValue()));
            }

            if (characteristic.getUuid().equals(FIRMWARE_CHARACTERISTIC_UUID)) {
                DefaultBLEDevice.this.info.put(BLE_DEVICE_INFO_FIRMWARE, new String(characteristic.getValue()).replace((char) 0x0, ' ').trim());
            }

            if (characteristic.getUuid().equals(HARDWARE_CHARACTERISTIC_UUID)) {
                DefaultBLEDevice.this.info.put(BLE_DEVICE_INFO_HARDWARE, new String(characteristic.getValue()));
            }

            if (characteristic.getUuid().equals(BATTERY_CHARACTERISTIC_UUID)) {
                DefaultBLEDevice.this.info.put(BLE_DEVICE_INFO_BATTERY, String.valueOf(characteristic.getIntValue(BluetoothGattCharacteristic.FORMAT_SINT8, 0)));
                fireEvent(EVENT_BATTERY_CHANGE, null);
                //修改设备信息已读取
                DefaultBLEDevice.this.isInfoRead = true;
                //发送设备信息读取成功事件
                fireEvent(EVENT_READ_INFO, null);
            }
        }

    }

    @Override
    public void readDeviceInfo() {
        // 读取设备信息
        read(INFO_SERVICE_UUID, INFO_CHARACTERISTIC_UUID);
        // 读取效验信息
        read(INFO_SERVICE_UUID, CHECKCODE_CHARACTERISTIC_UUID);
        // 读取固件信息
        read(INFO_SERVICE_UUID, FIRMWARE_CHARACTERISTIC_UUID);
        // 读取硬件信息
        read(INFO_SERVICE_UUID, HARDWARE_CHARACTERISTIC_UUID);
        // 读取电池信息
        BluetoothGattCharacteristic bluetoothGattCharacteristic
                = getBluetoothGattCharacteristic(BELT_BATTERY_SERVICE_UUID, BATTERY_CHARACTERISTIC_UUID);

        if (bluetoothGattCharacteristic == null) {
            read(INSOLE_BATTERY_SERVICE_UUID, BATTERY_CHARACTERISTIC_UUID);
        } else {
            read(BELT_BATTERY_SERVICE_UUID, BATTERY_CHARACTERISTIC_UUID);
        }
    }
}
