package hk.advanpro.android.sdk.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * Created by jalin on 2017/8/2.
 */

public class DateUtils {
    /**
     *  判断两个日期是否在同一类型的范围内
     * @param field 给定的日历字段。Calendar.DATE, Calendar.MONTH, Calendar.YEAR
     * @param date1 日期
     * @param date2 日期
     * @return 结果
     */
    public static boolean isSame(int field, long date1, long date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTimeInMillis(date1);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTimeInMillis(date2);
        return isSame(field, cal1, cal2);
    }

    /**
     * 判断两个日期是否在同一类型的范围内
     *
     * @param field 给定的日历字段。Calendar.DATE, Calendar.MONTH, Calendar.YEAR
     * @param date1 日期
     * @param date2 日期
     * @return 结果
     */
    public static boolean isSame(int field, Date date1, Date date2) {
        if (date1 == null || date2 == null) {
            return false;
        }
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        return isSame(field, cal1, cal2);
    }

    /**
     * 判断两个日期是否在同一类型的范围内
     *
     * @param field 给定的日历字段。Calendar.DATE, Calendar.MONTH, Calendar.YEAR
     * @param date1 日期
     * @param date2 日期
     * @return 结果
     */
    public static boolean isSame(int field, Calendar date1, Calendar date2) {
        if (date1 == null || date2 == null) {
            return false;
        }
        switch (field) {
            case Calendar.DATE:
                return date1.get(Calendar.ERA) == date2.get(Calendar.ERA) &&
                        date1.get(Calendar.YEAR) == date2.get(Calendar.YEAR) &&
                        date1.get(Calendar.DAY_OF_YEAR) == date2.get(Calendar.DAY_OF_YEAR);
            case Calendar.MONTH:
                return date1.get(Calendar.ERA) == date2.get(Calendar.ERA) &&
                        date1.get(Calendar.YEAR) == date2.get(Calendar.YEAR) &&
                        date1.get(Calendar.MONTH) == date2.get(Calendar.MONTH);
            case Calendar.YEAR:
                return date1.get(Calendar.ERA) == date2.get(Calendar.ERA) &&
                        date1.get(Calendar.YEAR) == date2.get(Calendar.YEAR);
            default:
                return false;
        }
    }

    /**
     * 获取基于当前日期偏移后的日期
     *
     * @param date   当前日期
     * @param offset 偏移的天数，负数向前偏移，正数向后偏移
     * @return 结果
     */
    public static Date getDay(Date date, int offset) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, offset);
        return calendar.getTime();
    }

    /**
     * 获取基于当前日期偏移后的日期
     *
     * @param date   当前日期
     * @param offset 偏移的天数，负数向前偏移，正数向后偏移
     * @return 结果
     */
    public static long getDay(long date, int offset) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(date);
        calendar.add(Calendar.DATE, offset);
        return calendar.getTimeInMillis();
    }

    /**
     * 获取基于当前月份的偏移值
     *
     * @param date   日期
     * @param offset 当前月份的偏移值
     * @return 月份的偏移值
     */
    public static Date getMonth(Date date, int offset) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, offset);
        return calendar.getTime();
    }

    /**
     * 获取前一天日期
     *
     * @param date 日期
     * @return 前一天日期
     */
    public static Date getPreviousDay(Date date) {
        return getDay(date, -1);
    }

    /**
     * 获取前一天日期
     *
     * @param date 日期
     * @return 前一天日期
     */
    public static long getPreviousDay(long date) {
        return getDay(date, -1);
    }

    /**
     * 获取后一天日期
     *
     * @param date 日期
     * @return 后一天日期
     */
    public static Date getNextDay(Date date) {
        return getDay(date, 1);
    }

    /**
     * 获取后一天日期
     *
     * @param date 日期
     * @return 后一天日期
     */
    public static long getNextDay(long date) {
        return getDay(date, 1);
    }

    /**
     * 返回指定格式日期
     *
     * @param date    日期
     * @param pattern 解析
     * @return 字符串
     */
    public static String formatDate(Date date, String pattern) {
        return new SimpleDateFormat(pattern).format(date);
    }

    /**
     * 返回指定格式日期
     *
     * @param time    日期
     * @param pattern 解析
     * @return 字符串
     */
    public static String formatDate(long time, String pattern) {
        return new SimpleDateFormat(pattern).format(time);
    }

    /**
     * 字符串解析成日期
     *
     * @param string  符串
     * @param pattern 解析
     * @return 解析错误返回null
     */
    public static Date str2Date(String string, String pattern) {
        try {
            return new SimpleDateFormat(pattern).parse(string);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 获取当前日期对应是星期几
     *
     * @param date 当前日期
     * @return 与Calendar的星期字段相同，1表示星期天，以此类推
     */
    public static int getDayOfWeek(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.DAY_OF_WEEK);
    }

    /**
     * 获取指定日期的当天毫秒数
     *
     * @param date 日期
     * @return 当天毫秒数
     */
    public static long getMillisInDay(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        return c.get(Calendar.HOUR_OF_DAY) * 3600000 + c.get(Calendar.MINUTE) * 60000 +
                c.get(Calendar.SECOND) * 1000 + c.get(Calendar.MILLISECOND);
    }

    /**
     * 将字符串日期解析成Date对象
     *
     * @param date    字符串
     * @param pattern 解析
     * @return Date对象
     */
    public static Date parseStringDate(String date, String pattern) {
        try {
            return new SimpleDateFormat(pattern).parse(date);
        } catch (ParseException e) {
            throw new IllegalArgumentException("日期格式不对");
        }
    }

    /**
     * 两个日期相差的天数(B-A)
     * @param dateA 日期
     * @param dateB 日期
     * @return 相差的天数
     */
    public static int daysBetween(Date dateA, Date dateB) {
        Calendar a = getStartOfDay(dateA);
        Calendar b = getStartOfDay(dateB);
        return (int) ((b.getTimeInMillis() - a.getTimeInMillis()) / (24 * 3600000L));
    }

    /**
     * 两个日期相差的天数(B-A)
     *
     * @param dateA A月
     * @param dateB B月
     * @return 两个日期天数
     */
    public static int daysBetween(long dateA, long dateB) {
        dateA = getStartOfDay(dateA);
        dateB = getStartOfDay(dateB);
        return (int) ((dateB - dateA) / (24 * 3600000L));
    }

    /**
     * 两个日期相差的月数(B-A)
     *
     * @param dateA A月
     * @param dateB B月
     * @return 两个日期相差月数
     */
    public static int monthsBetween(Date dateA, Date dateB) {
        Calendar calendarA = Calendar.getInstance();
        Calendar calendarB = Calendar.getInstance();
        calendarA.setTime(dateA);
        calendarB.setTime(dateB);
        return (calendarB.get(Calendar.YEAR) - calendarA.get(Calendar.YEAR)) * 12 +
                (calendarB.get(Calendar.MONTH) - calendarA.get(Calendar.MONTH));
    }

    /**
     * 将此日期时间设置成0点整
     *
     * @param date 日期
     * @return 时间设置成0点整
     */
    public static Calendar getStartOfDay(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar;
    }

    /**
     * 将此日期时间设置成23:999
     *
     * @param date 日期
     * @return 时间设置成23:999
     */
    public static Calendar getEndOfDay(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        return calendar;
    }

    /**
     * 将此日期时间设置成0点整
     *
     * @param date 日期
     * @return 时间设置成0点整
     */
    public static long getStartOfDay(long date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date(date));
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTimeInMillis();
    }

    /**
     * 根据条件获取相应最大值，如此日期的月有多少天，Calendar.DATE
     *
     * @param date  日期的
     * @param field Calendar.DATE
     * @return 获取相应最大值
     */
    public static int getActualMaximum(Date date, int field) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.getActualMaximum(field);
    }

    /**
     * 获取日期是当月的第几天
     *
     * @param date 日期
     * @return 当月的第几天
     */
    public static int getDayOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.DATE);
    }

    /**
     * 获取日期是当年的第几月，1月为0
     *
     * @param date 日期
     * @return 当年的第几月，1月为0
     */
    public static int getMonthOfYear(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.MONTH);
    }

    /**
     * 获取日期所在月份的的第一天
     *
     * @param date 日期
     * @return 所在月份的的第一天
     */
    public static Date getStartOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DATE, 1);
        return calendar.getTime();
    }

    /**
     * 获取日期所在月份的的最后一天
     *
     * @param date 日期
     * @return 所在月份的最后一天
     */
    public static Date getEndOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        return calendar.getTime();
    }

    /**
     * 获取日期所在年的最后一天
     *
     * @param date 日期
     * @return 年的最后一天
     */
    public static Date getEndOfYear(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.MONTH, 11);
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        return calendar.getTime();
    }

    /**
     * 获取日期所在年的第一天
     *
     * @param date 日期
     * @return 所在年的第一天
     */
    public static Date getStartOfYear(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.MONTH, 0);
        calendar.set(Calendar.DATE, 1);
        return calendar.getTime();
    }

    public static Date parseISODate(String str) throws ParseException {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
        return df.parse(str);
    }

    public static Date parseUTCDate(String str) throws ParseException {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        return df.parse(str);
    }

    /**
     * 得到UTC时间，类型为字符串，格式为"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"
     * @param date 日期
     * @return 得到UTC时间
     */
    public static String formatDateToUTC(Date date) {
        return formatDateToUTC(date, "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
    }

    public static String formatDateToUTC(Date date, String fmt) {
        // 1、取得本地时间：
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        // 2、取得时间偏移量：
        int zoneOffset = cal.get(java.util.Calendar.ZONE_OFFSET);
        // 3、取得夏令时差：
        int dstOffset = cal.get(java.util.Calendar.DST_OFFSET);
        // 4、从本地时间里扣除这些差量，即可以取得UTC时间：
        cal.add(java.util.Calendar.MILLISECOND, -(zoneOffset + dstOffset));
        return new SimpleDateFormat(fmt).format(cal.getTimeInMillis());
    }
}
